%% demo_caraest.m
%% Illustration des caractristiques des estimateurs
%% biais, variance, convergence
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


% $$$ figure(1)
% $$$ set(gcf, 'Position', [635 390 560 420], 'resize', 'off',...
% $$$ 'PaperPosition', [0.25 2.5 8 8*42/56])
% $$$ 
% $$$ figure(2)
% $$$ set(gcf, 'Position', [10 390 560 420], 'resize', 'off',...
% $$$ 'PaperPosition', [0.25 2.5 8 8*42/56])
% $$$ 

rq = 1;
rn = 100;
N = 1024;
NZ = 50;

randn('seed', pi);
nn = randn(N, 1);
%q = [-.7, .3, 1.6, 10];
q = [-.7, 4];
z = q(ones(N, 1), :) + sqrt(rn) * nn(:, ones(1, 2));

qhat = cumsum(z) ./ (cumsum(ones(N, 2)) + rn / rq);
qmc = cumsum(z) ./ cumsum(ones(N, 2));


figure(1), clf
set(gcf, 'Windowstyle', 'Docked')

for k = 8:8:N
  subplot(2,1,1)
  plot((1:k)', qhat(1:k, :) , 'b', 'LineWidth', 1), hold on
  plot([1; N], [q;q], 'k-.', 'LineWidth', 1), hold off
  set(gca, 'XLim', [0 N], 'Ylim', [-5 5]), grid
  xlabel('nombre de mesures'), ylabel('estime') 
  title('(a) Estimateur MAP')

  subplot(2,1,2)
  plot((1:k)', qmc(1:k, :), 'b', 'LineWidth', 1), hold on
  plot([1; N], [q;q], 'k-.', 'LineWidth', 1), hold off
  set(gca, 'XLim', [0 N], 'Ylim', [-5 5]), grid
  xlabel('nombre de mesures'), ylabel('estime')
  title('(b) Estimateur MV')
  
  drawnow
  
end

disp('Taper <RC> pour poursuivre')
pause

figure(2), clf
set(gcf, 'Windowstyle', 'Docked')

subplot(2,1,1)
plot((1:NZ)', qhat(1:NZ, :), 'b', 'LineWidth', 1), hold on
plot([1; NZ], [q;q], 'k-.', 'LineWidth', 1), hold off
set(gca, 'XLim', [0 NZ], 'Ylim', [-5 15]), grid
xlabel('nombre de mesures'), ylabel('estime'), title('(a) Estimateur MAP')

subplot(2,1,2)
plot((1:NZ)', qmc(1:NZ, :), 'b', 'LineWidth', 1), hold on
plot([1; NZ], [q;q], 'k-.', 'LineWidth', 1), hold off
set(gca, 'XLim', [0 NZ], 'Ylim', [-5 15]), grid
xlabel('nombre de mesures'), ylabel('estime'), title('(b) Estimateur MV')

