% Dmonstration de l'utilisation de la fonction de corrlation pour
% estimer un retard. Application  l'chographie
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.


% Paramtres gnraux

Te = 1e-06 / 4;
K1 = 100;
imp = K1*[0 .5 1 .5 .0];

K2 = 15;
N2 = 256;
f1 = 1.3/N2;
f2 = 15/N2;
f = f1 + (1:N2)/N2 * (f2-f1);
chirp = K2 * sin(2 * pi .* f .* (0:N2-1));

N = 2^12;
t = (0:N-1)*Te;
d = 3e+08 * t / 2 /1000;

sigma1 = .2;
sigma2 = .1;

% Sans compression d'impulsion

figure(1), clf
set(gcf, 'Windowstyle', 'Docked')

Pow = sum(imp.^2) / length(imp);
subplot(1,2,1)
plot(t(1:length(imp)), imp), grid on
xlabel('Temps (s.)'), ylabel('Amplitude')
title(sprintf('IMPUSION, PUISSANCE = %5.2g', Pow))

disp('Taper <Enter> pour poursuivre')
pause

x = zeros(size(t));
h = figure(2); clf
set(gcf, 'Windowstyle', 'Docked')
aux = get(h, 'Position');
Lx = aux(3);


arret = 100;
while arret >= 20
  x(floor(N/4)) = (1/100) * (1+sigma1*randn);
  x(floor(4*N/7)) = (1/200) * (1+sigma1*randn);
  x(floor(7*N/9)) = (1/350) * (1+sigma1*randn);
   
  z = conv2(x, imp, 'same') + sigma2 * randn(size(x));
  
  
  plot(d, z), set(gca, 'YLim', [-.5 1]), grid on
  xlabel('Distance (mm)'), title('SIGNAL REU'), drawnow
  
  pos = get(h, 'currentpoint');
  arret = 100 * pos(1) / Lx;
end

disp('Taper <Enter> pour poursuivre')
pause

% Sans compression d'impulsion, chirp

figure(1)
set(gcf, 'Windowstyle', 'Docked')

Pow2 = sum(chirp.^2) / length(chirp);
subplot(1,2,2)
plot(t(1:length(chirp)), chirp), grid on
xlabel('Temps (s.)'), ylabel('Amplitude')
title(sprintf('CHIRP, PUISSANCE = %5.2g', Pow2))

disp('Taper <Enter> pour poursuivre')
pause

x = zeros(size(t));
h = figure(3); clf
set(gcf, 'Windowstyle', 'Docked')
aux = get(h, 'Position');
Lx = aux(3);

clf

arret = 100;
while arret >= 20
  x(floor(N/4)) = (1/100) * (1+sigma1*randn);
  x(floor(4*N/7)) = (1/200) * (1+sigma1*randn);
  x(floor(7*N/9)) = (1/350) * (1+sigma1*randn);
   
  z = conv2(x, chirp, 'same') + sigma2 * randn(size(x));
  
  plot(d, z), set(gca, 'YLim', [-.5 1]), grid on
  xlabel('Distance (mm)'), title('SIGNAL REU'), drawnow
  drawnow
  
  pos = get(h, 'currentpoint');
  arret = 100 * pos(1) / Lx;
end

disp('Taper <Enter> pour poursuivre')
pause

% Avec compression d'impulsion

x = zeros(size(t));
h = figure(3); clf
set(gcf, 'Windowstyle', 'Docked')


arret = 100;
while arret >= 20
  x(floor(N/4)) = (1/100) * (1+sigma1*randn);
  x(floor(4*N/7)) = (1/200) * (1+sigma1*randn);
  x(floor(7*N/9)) = (1/350) * (1+sigma1*randn);
   
  z = conv2(x, chirp, 'same') + sigma2 * randn(size(x));
  y = conv2(z, fliplr(chirp), 'full');
  y = y(N2:length(y));
  
  subplot(2,1,1)
  plot(d, z), set(gca, 'YLim', [-.5 1]), grid on
  xlabel('Distance (mm)'), title('SIGNAL REU'), drawnow
  
  subplot(2,1,2)
  plot(d, y), set(gca, 'YLim', [-100 300]), grid on
  xlabel('Distance (mm)'), title('SORTIE DU CORRLATEUR'), drawnow
  
  pos = get(h, 'currentpoint');
  arret = 100 * pos(1) / Lx;
end
