% demo_estRI.m
% dmonstration des proprits de l'estimateur d'une RI
%
%    Copyright (C) 2008-2014  Yves Goussard
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

% Initialisation

%Paramtres gnraux
TypeB = ['bruit blanc '
         'bruit color'];
Taille = [32; 256];
SNR = [20; 0];

lambda = [1; 10];
alpha = [0; 0.95; 0.99];

Tmax = max(Taille);

% Gnration de la RI
t = (0:15:465)';
alpha1 = -10;     % coeff de la premire exponentielle
T = .2;			  % Priode du cosinus
alpha2 = -15;     % coeff de la deuxime exponentielle
h = (exp(alpha1 * t / 1000) - exp(alpha2 * t / 1000) / 4).* ...
     cos(2 * pi * t / T / 1000);

figure(1), clf
set(gcf, 'Windowstyle', 'Docked')

plot(t, h, 'LineWidth', 2), grid
set(gca, 'Xlim', [0 500], 'Ylim', [-0.4 0.8])
xlabel('temps (ms.)'), ylabel('amplitude')
title('Rponse impulsionnelle du systme', 'FontSize', 14)

% Calcul de la TFD de la RI
N = 256;			% Nombre de points de la fft
H = fft(h, N); f = (0:N-1)' / N; I = f >= 0.5; f(I) = f(I) - 1 ;

% Filtre colorant l'entre
[b, a] = butter(4, .1);
F = (fft(3*b, N) ./ fft(a, N))';

figure(2), clf
set(gcf, 'Windowstyle', 'Docked')

plot(fftshift(f), fftshift(abs(H)), '-', ...
    fftshift(f), fftshift(abs(F)), '-.', 'LineWidth', 2)
set(gca, 'Xlim', [0 .5]); grid
xlabel('frquences rduites'), ylabel('Module de la TF')
title(['Rponses frquentielles du systme et du filtre colorant ' ...
       'l''entre'], 'FontSize', 14)

% Entres bruit blanc et bruit color
randn('seed', pi);
ent(:, 1) = randn(Tmax, 1);
ent(:, 2) = filter (b, a, ent(:, 1));

% Gnration du bruit d'observation et calcul de sa puissance
randn('seed', exp(1));
bo = randn(Tmax, 1); Pbo = bo' * bo;

% Menu
titre = 'Estimation d''une rponse impulsionnelle';

ITypeB = 1;
ITaille = 2;
ISNR = 1;
Ilambda = 1;
Ialpha = 1;

TODO = 0;

while TODO ~= 7
  item1 = ['Entre : ', TypeB(ITypeB, :)];
  item2 = ['Taille du signal : ', sprintf('%1.0f', Taille(ITaille)), ...
          ' chantillons'];
  item3 = ['SNR : ', sprintf('%1.0f', SNR(ISNR)), ' dB'];
  item4 = ['lambda = ', sprintf('%1.0f', lambda(Ilambda))];
  item5 = ['alpha = ', sprintf('%4.2f', alpha(Ialpha))];
  item6 = 'Estimer';
  item7 = 'Quitter';
  
  TODO = menu(titre, item1, item2, item3, item4, item5, item6, item7);
  
  switch TODO
   case 1
    ITypeB = menu('Type de l''entre', TypeB(1, :), TypeB(2, :));
    
   case 2
    ITaille = menu('Taille du signal d''entre', ...
                   sprintf('%1.0f chantillons', Taille(1)), ...
                   sprintf('%1.0f chantillons', Taille(2)));
    
   case 3
    ISNR = menu('Rapport signal--bruit', ...
                   sprintf('%1.0f dB', SNR(1)), ...
                   sprintf('%1.0f dB', SNR(2)));
    
   case 4
    Ilambda = menu('Paramtre de rgularisation', ...
                   sprintf('%1.0f', lambda(1)), ...
                   sprintf('%1.0f', lambda(2)));
    
   case 5
    Ialpha = menu('Corrlation entre chantillons', ...
                   sprintf('%4.2f', alpha(1)), ...
                   sprintf('%4.2f', alpha(2)), ...
                   sprintf('%4.2f', alpha(3)));
    
   case 6
    % Gnration de la sortie et calcul de sa puissance
    y = conv(h, ent(:, ITypeB)); y = y(1:Tmax); Py = y' * y;
    
    % Bruitage de la sortie
    K = sqrt(Py / Pbo) * 10^(-SNR(ISNR)/20);
    z = y + K * bo;
    
    % Mise des signaux  la taille choisie
    x = ent(1:Taille(ITaille), ITypeB);
    z = z(1:Taille(ITaille));
    
    % Estimation
    X = toeplitz(x, [x(1), zeros(1, length(h)-1)]);
    h_MV = inv(X'*X) * (X'*z);
    h_MAP = inv(X'*X + lambda(Ilambda) * ...
                inv(toeplitz(alpha(Ialpha).^(0:length(h)-1)))) * (X'*z);
    
    % Trac
    figure(3), clf
    set(gcf, 'Windowstyle', 'Docked')

    plot(t, h, '-', t, h_MV, '--', t, h_MAP, '-.', 'LineWidth', [2])
    grid, set(gca, 'Xlim', [0 500], 'Ylim', [-0.4 0.8])
    xlabel('temps (ms.)'), ylabel('amplitude')
    title('Rsultats de l''estimation', 'FontSize', 14)
    legend('RI vraie', 'RI estime par MV', 'RI estime par MAP')
  end
  
  
end

